from django_lare.models import Lare


class LareMiddleware(object):
    def process_request(self, request):
        request.lare = Lare(request)

    def process_response(self, request, response):
        if request.lare.is_enabled():
            response['X-LARE-VERSION'] = request.lare.version
        return response



class LareViewMixin(object):
    """
    View mixin that provides lare functionality
    """
    lare_current_namespace = ""  # should be declared at instantiation of View

    def dispatch(self, request, *args, **kwargs):
        request.lare.set_current_namespace(self.lare_current_namespace)
        result = super(LareViewMixin, self).dispatch(request, *args, **kwargs)
        return result


class DefaultLareViewMixin(LareViewMixin):
    lare_site = True
    lare_page = True
    lare_content = True
    lare_inner_content = True

    def dispatch(self, request, *args, **kwargs):
        request.lare.set_current_namespace(self.lare_current_namespace)
        matching_count = request.lare.get_matching_count()
        self.lare_site = matching_count <= 0
        self.lare_page = matching_count <= 1
        self.lare_content = matching_count <= 2
        self.lare_inner_content = matching_count <= 3
        result = super(LareViewMixin, self).dispatch(request, *args, **kwargs)
        return result

    def get_context_data(self, **kwargs):
        context = super(DefaultLareViewMixin, self).get_context_data(**kwargs)
        context.update({'lare_site': self.lare_site,
                        'lare_page': self.lare_page,
                        'lare_content': self.lare_content,
                        'lare_inner_content': self.lare_inner_content})
        return context



from django.views.generic import TemplateView

from .mixins import DefaultLareViewMixin


class Page1View(DefaultLareViewMixin, TemplateView):
    template_name = 'tests/page_1.html'
    lare_current_namespace = "Site1.Page1"

    def get_context_data(self, **kwargs):
        result = super(Page1View, self).get_context_data(**kwargs)
        if self.lare_site:
            result.update({'site_string': 'site_1'})
        if self.lare_page:
            result.update({'page_string': 'page_1'})
        return result


class Page1Content1View(DefaultLareViewMixin, TemplateView):
    template_name = 'tests/page_1_content_1.html'
    lare_current_namespace = "Site1.Page1.Content1"

    def get_context_data(self, **kwargs):
        result = super(Page1Content1View, self).get_context_data(**kwargs)
        if self.lare_site:
            result.update({'site_string': 'site_1'})
        if self.lare_page:
            result.update({'page_string': 'page_1'})
        if self.lare_content:
            result.update({'content_string': 'content_1'})
        return result


class Page1Content1InnerContent1View(DefaultLareViewMixin, TemplateView):
    template_name = 'tests/page_1_content_1_inner_content_1.html'
    lare_current_namespace = "Site1.Page1.Content1.InnerContent1"

    def get_context_data(self, **kwargs):
        result = super(Page1Content1InnerContent1View, self).get_context_data(**kwargs)
        if self.lare_site:
            result.update({'site_string': 'site_1'})
        if self.lare_page:
            result.update({'page_string': 'page_1'})
        if self.lare_content:
            result.update({'content_string': 'content_1'})
        if self.lare_inner_content:
            result.update({'inner_content_string': 'inner_con_tent_1'})
        return result


class Page1Content1InnerContent2View(DefaultLareViewMixin, TemplateView):
    template_name = 'tests/page_1_content_1_inner_content_2.html'
    lare_current_namespace = "Site1.Page1.Content1.InnerContent2"

    def get_context_data(self, **kwargs):
        result = super(Page1Content1InnerContent2View, self).get_context_data(**kwargs)
        if self.lare_site:
            result.update({'site_string': 'site_1'})
        if self.lare_page:
            result.update({'page_string': 'page_1'})
        if self.lare_content:
            result.update({'content_string': 'content_1'})
        if self.lare_content:
            result.update({'inner_content_string': 'inner_con_tent_2'})
        return result


class Page1Content2View(DefaultLareViewMixin, TemplateView):
    template_name = 'tests/page_1_content_2.html'
    lare_current_namespace = "Site1.Page1.Content2"

    def get_context_data(self, **kwargs):
        result = super(Page1Content2View, self).get_context_data(**kwargs)
        if self.lare_site:
            result.update({'site_string': 'site_1'})
        if self.lare_page:
            result.update({'page_string': 'page_1'})
        if self.lare_content:
            result.update({'content_string': 'content_2'})
        return result


class Page2View(DefaultLareViewMixin, TemplateView):
    template_name = 'tests/page_2.html'
    lare_current_namespace = "Site1.Page2"

    def get_context_data(self, **kwargs):
        result = super(Page2View, self).get_context_data(**kwargs)
        if self.lare_site:
            result.update({'site_string': 'site_1'})
        if self.lare_page:
            result.update({'page_string': 'page_2'})
        return result


class NoLareView(TemplateView):
    template_name = 'tests/no_lare_page.html'

    def get_context_data(self, **kwargs):
        result = super(NoLareView, self).get_context_data(**kwargs)
        result.update({'no_lare_page_string': 'no-lare-page'})
        return result



from __future__ import unicode_literals

from django.conf.urls import patterns, url
from django.core.urlresolvers import reverse
from django.test import TestCase, Client
from django_lare.models import Lare

from django_lare.test_views import *


urlpatterns = patterns('',
    url(r'^page1/$',                                Page1View.as_view(),                    name='page_1'),
    url(r'^page1/content1/$',                       Page1Content1View.as_view(),            name='content_1'),
    url(r'^page1/content1/inner_content1/$',        Page1Content1InnerContent1View.as_view(),            name='inner_content_1'),
    url(r'^page1/content1/inner_content2/$',        Page1Content1InnerContent2View.as_view(),            name='inner_content_2'),
    url(r'^page1/content2/$',                       Page1Content2View.as_view(),            name='content_2'),
    url(r'^page2/$',                                Page2View.as_view(),                    name='page_2'),
    url(r'^no-lare-page/$',                         NoLareView.as_view(),                  name='no_lare_page'),
)


class TestLareRequests(TestCase):

    urls = 'django_lare.tests'
    page_1_string = 'page_1'
    content_1_string = 'content_1'
    content_2_string = 'content_2'
    # underscore to prevent detecting content_1 as part of inner_content_1
    inner_content_1_string = 'inner_con_tent_1'
    inner_content_2_string = 'inner_con_tent_2'
    page_2_string = 'page_2'
    no_lare_page_string = 'no-lare-page'

    # testing page level namespace
    def test_page_1_no_lare(self):
        client = Client()
        response = client.get(reverse('page_1'))
        self.assertContains(response, self.page_1_string)
        self.assertContains(response, '<html>')
        self.assertNotContains(response, '<lare-body>')
        self.assertContains(response, 'Site1.Page1')

    def test_page_1_lare_with_namespace(self):
        client = Client()
        response = client.get(reverse('page_1'), **{'HTTP_X_LARE': 'Site1.Page2'})
        self.assertContains(response, self.page_1_string)
        self.assertContains(response, '<lare-body>')
        self.assertNotContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1')

    def test_page_1_lare_different_namespace(self):
        client = Client()
        response = client.get(reverse('page_1'), **{'HTTP_X_LARE': 'Site2'})
        self.assertContains(response, self.page_1_string)
        self.assertNotContains(response, '<lare-body>')
        self.assertContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1')

    # testing content level namespace
    def test_content_1_no_lare(self):
        client = Client()
        response = client.get(reverse('content_1'))
        self.assertContains(response, self.page_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertContains(response, '<html>')
        self.assertNotContains(response, '<lare-body>')
        self.assertContains(response, 'Site1.Page1.Content1')

    def test_content_1_lare_current_namespace(self):
        client = Client()
        response = client.get(reverse('content_1'), **{'HTTP_X_LARE': 'Site1.Page1.Content1'})
        self.assertNotContains(response, self.page_1_string)
        self.assertNotContains(response, self.content_1_string)
        self.assertContains(response, '<lare-body>')
        self.assertNotContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1')

    def test_content_1_lare_page_namespace(self):
        client = Client()
        response = client.get(reverse('content_1'), **{'HTTP_X_LARE': 'Site1.Page1'})
        self.assertNotContains(response, self.page_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertContains(response, '<lare-body>')
        self.assertNotContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1')

    def test_content_1_lare_content_namespace(self):
        client = Client()
        response = client.get(reverse('content_1'), **{'HTTP_X_LARE': 'Site1.Page1.Content2'})
        self.assertNotContains(response, self.page_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertContains(response, '<lare-body>')
        self.assertNotContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1')

    def test_content_1_lare_different_page_namespace(self):
        client = Client()
        response = client.get(reverse('content_1'), **{'HTTP_X_LARE': 'Site1.Page2'})
        self.assertContains(response, self.page_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertContains(response, '<lare-body>')
        self.assertNotContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1')

    def test_content_1_lare_different_site_namespace(self):
        client = Client()
        response = client.get(reverse('content_1'), **{'HTTP_X_LARE': 'Site2.Page1'})
        self.assertContains(response, self.page_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertNotContains(response, '<lare-body>')
        self.assertContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1')

    # testing inner_content level namespace
    def test_inner_content_1_no_lare(self):
        client = Client()
        response = client.get(reverse('inner_content_1'))
        self.assertContains(response, self.inner_content_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertContains(response, self.page_1_string)
        self.assertContains(response, '<html>')
        self.assertNotContains(response, '<lare-body>')
        self.assertContains(response, 'Site1.Page1.Content1.InnerContent1')

    def test_inner_content_1_lare_with_namespace(self):
        client = Client()
        response = client.get(reverse('inner_content_1'), **{'HTTP_X_LARE': 'Site1.Page1.Content1.InnerContent2'})
        self.assertContains(response, self.inner_content_1_string)
        self.assertNotContains(response, self.content_1_string)
        self.assertNotContains(response, self.page_1_string)
        self.assertContains(response, '<lare-body>')
        self.assertNotContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1.InnerContent1')

    def test_inner_content_1_lare_different_content_namespace(self):
        client = Client()
        response = client.get(reverse('inner_content_1'), **{'HTTP_X_LARE': 'Site1.Page1.Content2'})
        self.assertContains(response, self.inner_content_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertNotContains(response, self.page_1_string)
        self.assertContains(response, '<lare-body>')
        self.assertNotContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1.InnerContent1')

    def test_inner_content_1_lare_different_page_namespace(self):
        client = Client()
        response = client.get(reverse('inner_content_1'), **{'HTTP_X_LARE': 'Site1.Page2'})
        self.assertContains(response, self.inner_content_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertContains(response, self.page_1_string)
        self.assertContains(response, '<lare-body>')
        self.assertNotContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1.InnerContent1')

    def test_inner_content_1_lare_different_site_namespace(self):
        client = Client()
        response = client.get(reverse('inner_content_1'), **{'HTTP_X_LARE': 'Site2.Page1.Content1'})
        self.assertContains(response, self.inner_content_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertContains(response, self.page_1_string)
        self.assertNotContains(response, '<lare-body>')
        self.assertContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1.InnerContent1')

    # testing old version
    def test_old_version(self):
        versions = Lare.supported_version.split('.')
        versions[len(versions)-1] = "{0}".format(int(versions[len(versions)-1])-1)
        version = '.'.join(versions)
        client = Client()
        response = client.get(reverse('inner_content_1'), **{'HTTP_X_LARE_VERSION': version, 'HTTP_X_LARE': 'Site1.Page1.Content2'})
        self.assertContains(response, self.inner_content_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertContains(response, self.page_1_string)
        self.assertNotContains(response, '<lare-body>')
        self.assertContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1.InnerContent1')

    def test_future_version(self):
        versions = Lare.supported_version.split('.')
        versions[len(versions)-1] = "{0}".format(int(versions[len(versions)-1])+1)
        version = '.'.join(versions)
        client = Client()
        response = client.get(reverse('inner_content_1'), **{'HTTP_X_LARE_VERSION': version, 'HTTP_X_LARE': 'Site1.Page1.Content2'})
        self.assertContains(response, self.inner_content_1_string)
        self.assertContains(response, self.content_1_string)
        self.assertNotContains(response, self.page_1_string)
        self.assertContains(response, '<lare-body>')
        self.assertNotContains(response, '<html>')
        self.assertContains(response, 'Site1.Page1.Content1.InnerContent1')

    # testing non lare page
    def test_non_lare_page(self):
        client = Client()
        response = client.get(reverse('no_lare_page'))
        self.assertContains(response, self.no_lare_page_string)
        self.assertContains(response, '<html>')
        self.assertNotContains(response, '<lare-body>')

    def test_non_lare_page_with_namespace(self):
        client = Client()
        response = client.get(reverse('no_lare_page'), **{'HTTP_X_LARE': 'Site2.Page1.Content1'})
        self.assertContains(response, self.no_lare_page_string)
        self.assertContains(response, '<html>')
        self.assertNotContains(response, '<lare-body>')
